// novaPrintDlg.cpp : implementation file
//
#include "stdafx.h"
#include "novaPrint.h"
#include "novaPrintDlg.h"

//include novaPDF headers
#include "..\..\..\include\nverrors.h"
#include "..\..\..\include\novamessages.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#define PRINTER_NAME        L"novaPDF SDK 9"
#define SMALL_SIZE_PROFILE  L"Small Size Profile"
#define FULL_OPT_PROFILE    L"Full Options Profile"
#define PROFILE_IS_PUBLIC   0

//register File Saved message for novaPDF Printer
const UINT  wm_Nova_FileSaved = RegisterWindowMessageW( MSG_NOVAPDF2_FILESAVED );
const UINT  wm_Nova_PrintError = RegisterWindowMessageW( MSG_NOVAPDF2_PRINTERROR );

// CAboutDlg dialog used for App About
class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

	// Dialog Data
	enum { IDD = IDD_ABOUTBOX };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

	// Implementation
protected:
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()

// CnovaPrintDlg dialog
CnovaPrintDlg::CnovaPrintDlg(CWnd* pParent /*=NULL*/)
: CDialog(CnovaPrintDlg::IDD, pParent)
, m_bCompress(FALSE)
, m_bEmbed(FALSE)
, m_bDetect(FALSE)
, m_strTitle(_T(""))
, m_strSubject(_T(""))
, m_strPdfFile(_T(""))
, m_strFileToConvert(_T(""))
, m_strProfileId(NULL)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_bPrintJobPending = FALSE;
	m_wsProfileSmall = NULL;
	m_wsProfileFull = NULL;
	// initialize COM libraries
	HRESULT hr = CoInitialize(NULL);
	if (FAILED (hr)) { 
		::MessageBoxW(NULL, L"Failed to initialize COM", L"novaPDF", MB_OK); 
	} 
}

CnovaPrintDlg::~CnovaPrintDlg()
{
	HRESULT hr;
	//delete profiles
	if (m_wsProfileSmall)
	{
		hr = m_novaOptions->DeleteProfile(m_wsProfileSmall);
		CoTaskMemFree(m_wsProfileSmall);
	}
	if (m_wsProfileFull)
	{
		hr = m_novaOptions->DeleteProfile(m_wsProfileFull);
		CoTaskMemFree(m_wsProfileFull);
	}
	// destroy our nova options object
	if (m_novaOptions) 
	{
		m_novaOptions->Release();
	}
	// uninitialize COM libraries
	CoUninitialize();
}

void CnovaPrintDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Check(pDX, IDC_CK_COMPRESS, m_bCompress);
	DDX_Check(pDX, IDC_CK_EMBED, m_bEmbed);
	DDX_Check(pDX, IDC_CK_DETECT, m_bDetect);
	DDX_Text(pDX, IDC_ED_TITLE, m_strTitle);
	DDX_Text(pDX, IDC_ED_SUBJECT, m_strSubject);
	DDX_Text(pDX, IDC_ED_FILENAME, m_strPdfFile);
	DDX_Text(pDX, IDC_ED_FILE_TO_CONVERT, m_strFileToConvert);
}

BEGIN_MESSAGE_MAP(CnovaPrintDlg, CDialog)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_CBN_SELCHANGE(IDC_CMB_PROFILES, OnCbnSelchangeCmbProfiles)
	ON_BN_CLICKED(IDC_BTN_BROWSE, OnBnClickedBtnBrowse)
	ON_REGISTERED_MESSAGE(wm_Nova_FileSaved, OnNovaPDFFileSaved)
	ON_REGISTERED_MESSAGE(wm_Nova_PrintError, OnNovaPDFPrintError)
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
END_MESSAGE_MAP()

// CnovaPrintDlg message handlers
BOOL CnovaPrintDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);         // Set big icon
	SetIcon(m_hIcon, FALSE);        // Set small icon

	// TODO: Add extra initialization here
	HRESULT hr = S_OK;
	m_novaOptions = 0;

	//create an instance of INovaPdfOptions
	hr = CoCreateInstance(__uuidof(NovaPdfOptions90), NULL, CLSCTX_INPROC_SERVER, __uuidof(INovaPdfOptions90), (LPVOID*) &m_novaOptions);
	if (SUCCEEDED(hr)) 
	{
		// initialize the NovaPdfOptions object to use with a printer licensed for SDK
        // if you have an application license for novaPDF SDK, 
        // pass the license key to the Initialize() function
		hr = m_novaOptions->Initialize(PRINTER_NAME, L"");
	}
	else 
	{
		::MessageBoxW(NULL, L"Failed to create novaPDF COM object", L"novaPDF", MB_OK); 
	}

	// get the directory of the application to save the PDF fles there
	WCHAR szExeDirectory[MAX_PATH] = L"";
	WCHAR drive[MAX_PATH], dir[MAX_PATH];
	GetModuleFileNameW(NULL, szExeDirectory, MAX_PATH);
	_wsplitpath(szExeDirectory, drive, dir, NULL, NULL);
	wcscpy(szExeDirectory, drive);
	wcscat(szExeDirectory, dir);

	// Add a profile called "Small size". If profile L"Small size" exists this will fail
	hr = m_novaOptions->AddProfile(SMALL_SIZE_PROFILE, PROFILE_IS_PUBLIC, &m_wsProfileSmall);

	//load the newly created profile
	if (SUCCEEDED(hr) && m_wsProfileSmall) 
	{
		//load profile
		m_novaOptions->LoadProfile(m_wsProfileSmall);

		// disable the "Save PDF file as" prompt 
		m_novaOptions->SetOptionLong(NOVAPDF_SAVE_PROMPT_TYPE, PROMPT_SAVE_NONE);
		// set generated Pdf files destination folder ("c:\")
		m_novaOptions->SetOptionLong(NOVAPDF_SAVE_LOCATION, LOCATION_TYPE_LOCAL);
		m_novaOptions->SetOptionLong(NOVAPDF_SAVE_FOLDER_TYPE, SAVEFOLDER_CUSTOM);
		m_novaOptions->SetOptionString(NOVAPDF_SAVE_FOLDER, szExeDirectory);
		// set output file name
		m_novaOptions->SetOptionString(NOVAPDF_SAVE_FILE_NAME, L"PDF Converter small size.pdf");
		// if file exists in the destination folder, append a counter to the end of the file name
		m_novaOptions->SetOptionLong(NOVAPDF_SAVE_FILEEXIST_ACTION, FILE_CONFLICT_STRATEGY_AUTONUMBER_NEW);

		// don't detect URLs
		m_novaOptions->SetOptionBool(NOVAPDF_URL_ANALIZE, FALSE);

		// Set image compresion method to JPEG and quality to 75, possible values are from 10 to 100
		m_novaOptions->SetOptionBool(NOVAPDF_COMPRESS_ENABLE, TRUE);
		m_novaOptions->SetOptionBool(NOVAPDF_COMPRESS_HIGH_COLOR, TRUE);
		m_novaOptions->SetOptionLong(NOVAPDF_COMPRESS_HIGH_COLOR_TYPE, COMPRESS_METHOD_JPEG);
		m_novaOptions->SetOptionLong(NOVAPDF_COMPRESS_HIGH_COLOR_LEVEL, 75);

		// make sure text compression is enabled, and set compression level to 9 (maximum) (posible values are 1-9)
		m_novaOptions->SetOptionBool(NOVAPDF_COMPRESS_TEXT_GRAPHICS, TRUE);
		m_novaOptions->SetOptionLong(NOVAPDF_COMPRESS_TEXT_GRAPHICS_LEVEL, 9);

		// disable unused font embeding
		m_novaOptions->SetOptionBool(NOVAPDF_FONTS_EMBED_ALL_USED_FONTS, FALSE);
		m_novaOptions->ClearFontOptions();

		//set document subject
		m_novaOptions->SetOptionString(NOVAPDF_DOCINFO_SUBJECT, L"My Subject - small file");

		//save profile changes
		m_novaOptions->SaveProfile();
	}

	// Add the profile "Full options", and edit its options
	hr = m_novaOptions->AddProfile(FULL_OPT_PROFILE, PROFILE_IS_PUBLIC, &m_wsProfileFull);

	//load the newly created profile
	if (SUCCEEDED(hr) && m_wsProfileFull) 
	{
		//load profile
		hr = m_novaOptions->LoadProfile(m_wsProfileFull);

		// disable the "Save PDF file as" prompt 
		m_novaOptions->SetOptionLong(NOVAPDF_SAVE_PROMPT_TYPE, PROMPT_SAVE_NONE);
		// set generated Pdf files destination folder ("c:\")
		m_novaOptions->SetOptionLong(NOVAPDF_SAVE_LOCATION, LOCATION_TYPE_LOCAL);
		m_novaOptions->SetOptionLong(NOVAPDF_SAVE_FOLDER_TYPE, SAVEFOLDER_CUSTOM);
		m_novaOptions->SetOptionString(NOVAPDF_SAVE_FOLDER, szExeDirectory);
		// set output file name "[N] full", the [N] macro means document name
		m_novaOptions->SetOptionString(NOVAPDF_SAVE_FILE_NAME, L"[N] full");
		// if file exists in the destination folder, append a counter to the end of the file name
		m_novaOptions->SetOptionLong(NOVAPDF_SAVE_FILEEXIST_ACTION, FILE_CONFLICT_STRATEGY_AUTONUMBER_NEW);

		// enable URL detection
		m_novaOptions->SetOptionBool(NOVAPDF_URL_ANALIZE, TRUE);

		// set image JPEG quality to 100 (maximum)
		m_novaOptions->SetOptionBool(NOVAPDF_COMPRESS_ENABLE, TRUE);
		m_novaOptions->SetOptionBool(NOVAPDF_COMPRESS_HIGH_COLOR, TRUE);
		m_novaOptions->SetOptionLong(NOVAPDF_COMPRESS_HIGH_COLOR_TYPE, COMPRESS_METHOD_JPEG);
		m_novaOptions->SetOptionLong(NOVAPDF_COMPRESS_HIGH_COLOR_LEVEL, 100);

		// set text compression to 6 (on the "zip" scale from 0 to 9)
		m_novaOptions->SetOptionBool(NOVAPDF_COMPRESS_TEXT_GRAPHICS, TRUE);
		m_novaOptions->SetOptionLong(NOVAPDF_COMPRESS_TEXT_GRAPHICS_LEVEL, 6);

		// enable font files embedding
		m_novaOptions->SetOptionBool(NOVAPDF_FONTS_EMBED_ALL_USED_FONTS, TRUE);
		m_novaOptions->ClearFontOptions();

		//set document subject
		m_novaOptions->SetOptionString(NOVAPDF_DOCINFO_SUBJECT, L"My Subject - full");
		 
		//save profile changes
		hr = m_novaOptions->SaveProfile();
	}

	// Fill profiles combo box with profile names
	m_strProfileId = NULL;
	LPWSTR pwsProfileName = NULL;
	int nIndex;
	CComboBox *pCombo = static_cast<CComboBox*>(GetDlgItem(IDC_CMB_PROFILES));
	hr = m_novaOptions->GetFirstProfile(&m_strProfileId);
	while (SUCCEEDED(hr) && hr != NV_NO_MORE_PROFILES)
	{
		if (m_strProfileId)
		{
			m_novaOptions->LoadProfile(m_strProfileId);
			m_novaOptions->GetOptionString(NOVAPDF_PROFILE_NAME, &pwsProfileName);
			// add profile name to combo-box
			nIndex = pCombo->AddString(pwsProfileName);
			pCombo->SetItemDataPtr(nIndex, m_strProfileId);
			CoTaskMemFree(pwsProfileName);
		}
		// get next profile if it exists
		m_strProfileId = NULL;
		hr = m_novaOptions->GetNextProfile(&m_strProfileId);
	}

	// select the active profile in the profiles combo box
	LPWSTR strActiveProfileId = NULL;
	LPWSTR strActiveProfileName = NULL;
	hr = m_novaOptions->GetActiveProfile(&strActiveProfileId);
	if (SUCCEEDED(hr)) 
	{
		m_strProfileId = strActiveProfileId;
	}
	else
	{
		//if no active profile, use the Full Options profile
		m_strProfileId = m_wsProfileFull;
	}
	m_novaOptions->LoadProfile(m_strProfileId);
	m_novaOptions->GetOptionString(NOVAPDF_PROFILE_NAME, &strActiveProfileName);
	pCombo->SelectString(-1, strActiveProfileName);
	CoTaskMemFree(strActiveProfileName);

	UpdateDialogFromProfile(m_strProfileId);
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CnovaPrintDlg::UpdateDialogFromProfile(LPWSTR pstrProfileId)
{
	UpdateData(TRUE);
	// update dialog data with the data from the profile
	HRESULT hr = S_OK;
	BOOL bVal;

	//load profile
	m_novaOptions->LoadProfile(pstrProfileId);

	m_novaOptions->GetOptionBool(NOVAPDF_COMPRESS_TEXT_GRAPHICS, &m_bCompress);
	m_novaOptions->GetOptionBool(NOVAPDF_URL_ANALIZE, &m_bDetect);
	m_novaOptions->GetOptionBool(NOVAPDF_FONTS_EMBED_ALL_USED_FONTS, &m_bEmbed);
	
	LPWSTR strOption = 0;
	hr = m_novaOptions->GetOptionString(NOVAPDF_SAVE_FILE_NAME, &strOption);
	if (SUCCEEDED(hr)) 
	{
		m_strPdfFile = strOption;
		CoTaskMemFree(strOption);
	}

	hr = m_novaOptions->GetOptionString(NOVAPDF_DOCINFO_SUBJECT, &strOption);
	if (SUCCEEDED(hr)) 
	{
		m_strSubject = strOption;
		CoTaskMemFree(strOption);
	}

	hr = m_novaOptions->GetOptionString(NOVAPDF_DOCINFO_TITLE, &strOption);
	if (SUCCEEDED(hr)) 
	{
		m_strTitle = strOption;
		CoTaskMemFree(strOption);
	}

	UpdateData(FALSE);
}

void CnovaPrintDlg::UpdateProfileFromDialog()
{
	UpdateData(TRUE);
	HRESULT hr = S_OK;
	//load profile
	m_novaOptions->LoadProfile(m_strProfileId);
	//set options
	m_novaOptions->SetOptionBool(NOVAPDF_COMPRESS_TEXT_GRAPHICS, m_bCompress);
	m_novaOptions->SetOptionBool(NOVAPDF_URL_ANALIZE, m_bDetect);
	m_novaOptions->SetOptionBool(NOVAPDF_FONTS_EMBED_ALL_USED_FONTS, m_bEmbed);
	m_novaOptions->SetOptionString(NOVAPDF_SAVE_FILE_NAME, m_strPdfFile);
	m_novaOptions->SetOptionString(NOVAPDF_DOCINFO_SUBJECT, m_strSubject);
	m_novaOptions->SetOptionString(NOVAPDF_DOCINFO_TITLE, m_strTitle);
	// mark finish changing options so they are saved for the printer
	m_novaOptions->SaveProfile();
}

void CnovaPrintDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CnovaPrintDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CnovaPrintDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CnovaPrintDlg::OnBnClickedOk()
{
	if (m_bPrintJobPending)
	{
		MessageBox(L"A print job is pending", L"novaPDF", MB_OK);
		return;
	}

	CWaitCursor wait;
	// save settings from dialog to profile
	UpdateProfileFromDialog();

	// check to see if there is a file to convert
	int nResult = _waccess(m_strFileToConvert, 0);
	if (nResult != 0)
	{
		// no file to convert
		MessageBox(L"File to convert not found !");
		return; // OnOK();
	}

	HRESULT hr = S_OK;

	// set the active profile to be used for printing
	hr = m_novaOptions->SetActiveProfile(m_strProfileId);

	// register our window to receive messages from the printer
#pragma warning (disable : 4311)
	hr = m_novaOptions->RegisterEventWindow((LONG) GetSafeHwnd());
#pragma warning (default : 4311)

	// set novaPDF as default printer, so it will be used by ShellExecute
	hr = m_novaOptions->SetDefaultPrinter();

	// license file for ShellExecute
	//hr = m_novaOptions->LicenseShellExecuteFile(m_strFileToConvert.AllocSysString());

	// print the document
	m_bPrintJobPending = TRUE;
	HINSTANCE hExec = ShellExecute(GetSafeHwnd(), L"print", m_strFileToConvert, NULL, NULL, SW_HIDE);

#pragma warning (disable : 4311)
	if ((int)hExec <= 32) // failed to execute program
#pragma warning (default : 4311)
	{
		m_bPrintJobPending = FALSE;
		m_novaOptions->UnRegisterEventWindow();
		m_novaOptions->RestoreDefaultPrinter();
	}

}

void CnovaPrintDlg::OnCbnSelchangeCmbProfiles()
{
	// save dialog control values to data members
	UpdateData(TRUE);

	CComboBox *pCombo = static_cast<CComboBox*>(GetDlgItem(IDC_CMB_PROFILES));
	int nIndex = pCombo->GetCurSel();
	if (nIndex >= 0)
	{
		m_strProfileId = (LPWSTR)pCombo->GetItemDataPtr( nIndex );
		// activate the profile that was selected in the combo box
		UpdateDialogFromProfile(m_strProfileId);
		m_novaOptions->SetActiveProfile(m_strProfileId);
	}
}

void CnovaPrintDlg::OnBnClickedBtnBrowse()
{
	CFileDialog dlg(TRUE, NULL, NULL, 0, NULL, (CWnd*) this, 0);
	if (IDOK == dlg.DoModal())
	{
		UpdateData(TRUE);
		m_strFileToConvert = dlg.GetPathName();
		UpdateData(FALSE);
	}
}

LRESULT CnovaPrintDlg::OnNovaPDFFileSaved(WPARAM wParam, LPARAM lParam)
{
	// restore original default printer
	m_novaOptions->UnRegisterEventWindow();
	m_novaOptions->RestoreDefaultPrinter();
	m_bPrintJobPending = FALSE;
	return 0;
}

LRESULT CnovaPrintDlg::OnNovaPDFPrintError(WPARAM wParam, LPARAM lParam)
{
	switch(wParam)
	{
	case ERROR_MSG_TEMP_FILE:
		MessageBox(L"Error saving temporary file on printer server", L"novaPDF", MB_OK);
		break;
	case ERROR_MSG_LIC_INFO:
		MessageBox(L"Error reading license information", L"novaPDF", MB_OK);
		break;
	case ERROR_MSG_SAVE_PDF:
		MessageBox(L"Error saving PDF file", L"novaPDF", MB_OK);
		break;
	case ERROR_MSG_JOB_CANCELED:
		MessageBox(L"Print job was canceled", L"novaPDF", MB_OK);
		break;
	}
	// restore original default printer
	m_novaOptions->UnRegisterEventWindow();
	m_novaOptions->RestoreDefaultPrinter();
	m_bPrintJobPending = FALSE;
	return 0;
}

void CnovaPrintDlg::OnBnClickedCancel()
{
	if (m_bPrintJobPending)
	{
		if(IDYES == MessageBox(L"A print job is pending. Are you sure you want to close the application?", L"novaPDF", MB_YESNO))
		{
			OnCancel();
		}
	}
	else
	{
		OnCancel();
	}
}
